# Objective of this script is to fix some files generated by Pigeon because Pigeon does not support Custom Codecs.

echo "Generate Pigeon Files."
(cd .. && dart run pigeon --input ./pigeons/messages.dart)
echo "Generation complete."
echo "First formatting."
melos format > /dev/null
echo "Formatting complete."

# # Fix Java files
FILE_NAME="../../cloud_firestore/android/src/main/java/io/flutter/plugins/firebase/firestore/GeneratedAndroidFirebaseFirestore.java"
sed -i '' 's/ArrayList<Object> toList() {/public ArrayList<Object> toList() {/' "$FILE_NAME"
sed -i '' 's/private static class FirebaseFirestoreHostApiCodec extends StandardMessageCodec {/private static class FirebaseFirestoreHostApiCodec extends FlutterFirebaseFirestoreMessageCodec {/' "$FILE_NAME"

echo "Android modification complete."

# Fix iOS files
FILE_NAME="../../cloud_firestore/ios/Classes/FirestoreMessages.g.m"
sed -i '' '/#import "FirestoreMessages.g.h"/a\
#import "FLTFirebaseFirestoreReader.h"\
#import "FLTFirebaseFirestoreWriter.h"
' $FILE_NAME
sed -i '' 's/pigeonResult.newIndex = newIndex;/pigeonResult.index = newIndex;/' $FILE_NAME
sed -i '' -e 's/pigeonResult.newIndex = GetNullableObjectAtIndex(list, 3);/pigeonResult.index = GetNullableObjectAtIndex(list, 3);/' -e 's/NSAssert(pigeonResult.newIndex != nil, @"");/NSAssert(pigeonResult.index != nil, @"");/' $FILE_NAME
sed -i '' 's/(self\.newIndex \?: \[NSNull null\]),/(self.index ?: [NSNull null]),/' $FILE_NAME
sed -i '' 's/@interface FirebaseFirestoreHostApiCodecReader : FlutterStandardReader/@interface FirebaseFirestoreHostApiCodecReader : FLTFirebaseFirestoreReader/' $FILE_NAME
sed -i '' 's/@interface FirebaseFirestoreHostApiCodecWriter : FlutterStandardWriter/@interface FirebaseFirestoreHostApiCodecWriter : FLTFirebaseFirestoreWriter/' $FILE_NAME

FILE_NAME="../../cloud_firestore/ios/Classes/Public/FirestoreMessages.g.h"
sed -i '' 's/@property(nonatomic, strong) NSNumber \*newIndex;/@property(nonatomic, strong) NSNumber \*index;/' $FILE_NAME

echo "iOS modification complete."

# Fix Windows files
FILE_NAME="../../cloud_firestore/windows/messages.g.h"
sed -i '' '/#include <string>/a\
#include "firestore_codec.h"
' $FILE_NAME
perl -i -0777 -pe 's|private:\n(\s+static PigeonSnapshotMetadata FromEncodableList\(\n\s+const flutter::EncodableList& list\);\n\s+flutter::EncodableList ToEncodableList\(\) const;)|\1\n\n private:|gs' $FILE_NAME
perl -0777 -i -pe 's/private:\n(\x20{2}static PigeonDocumentSnapshot FromEncodableList\(\n\x20{6}const flutter::EncodableList& list\);\n\x20{2}flutter::EncodableList ToEncodableList\(\) const;)/\1\n\n private:/gs' $FILE_NAME
perl -0777 -i -pe 's/private:\n(\x20{2}static PigeonDocumentChange FromEncodableList\(\n\x20{6}const flutter::EncodableList& list\);\n\x20{2}flutter::EncodableList ToEncodableList\(\) const;)/\1\n\n private:/gs' $FILE_NAME

sed -i '' 's/: public flutter::StandardCodecSerializer {/: public cloud_firestore_windows::FirestoreCodec {/' $FILE_NAME

FILE_NAME="../../cloud_firestore/windows/messages.g.cpp"
sed -i '' 's/flutter::StandardCodecSerializer::WriteValue(value, stream);/cloud_firestore_windows::FirestoreCodec::WriteValue(value, stream);/' $FILE_NAME
sed -i '' 's/return flutter::StandardCodecSerializer::ReadValueOfType(type, stream);/return cloud_firestore_windows::FirestoreCodec::ReadValueOfType(type, stream);/' $FILE_NAME

echo "Windows modification complete."

# Fix Dart files
FILE_NAME="../lib/src/pigeon/messages.pigeon.dart"
sed -i '' '/import '\''package:flutter\/foundation\.dart'\'' show ReadBuffer, WriteBuffer;/i\
import '\''package:cloud_firestore_platform_interface/src/method_channel/utils/firestore_message_codec.dart'\'';
' $FILE_NAME
sed -i '' 's/class _FirebaseFirestoreHostApiCodec extends StandardMessageCodec {/class _FirebaseFirestoreHostApiCodec extends FirestoreMessageCodec {/' $FILE_NAME
(cd .. && dart fix --apply > /dev/null) 

FILE_NAME="../test/pigeon/test_api.dart"
sed -i '' -E 's/import '\''dart:typed_data'\'' show Float64List, Int32List, Int64List, Uint8List;/import '\''dart:typed_data'\'' show Uint8List;/' $FILE_NAME

echo "Dart modification complete."

echo "Final formatting."
melos format > /dev/null
echo "Formatting complete."

echo "All modifications complete."
